/*
 * RGB.c
 *
 * Created: 01.02.2014 0:26:14
 *  Author: Сергей
 */ 
#define F_CPU 8000000


#include <avr/io.h>
#include <avr/iom48.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>


unsigned char e_scales EEMEM =200;					  //Инициализация переменных в EEPROM
unsigned char e_red1   EEMEM =200;
unsigned char e_green1 EEMEM =0;
unsigned char e_blue1  EEMEM =0;
unsigned char e_tmp    EEMEM =1;



unsigned char NewState, OldState, upState, downState; //переменные для обработки энкодера 
unsigned char tmp;                                    //переменная для работы функций смены цветов  
unsigned char set;                                    //переменная для хранения режима работы регулятора
unsigned int  ReturnCounter;                          //программный счетчик для отсчета времени возврата на первый режим регулировки
unsigned int  SaveCounter;                            //программный счетчик для отсчета времени задержки сохранения данных в еепром 
unsigned char red1,green1,blue1,scales;               //буфер значений шим для динамических эффектов

/* Придаем символьные имена регистрам, отвечающим за ШИМ */
#define RED1    OCR1A                 //PB1
#define GREEN1  OCR0A                 //PD6
#define BLUE1   OCR0B                 //PD5

#define SCALES  OCR1BL                //PB2



#define STEP               10        //шаг регулировки яркости каналов
#define LIGHT_UP           5         //скорость начального плавного розжига
#define LIGHT_DOWN         10        //скорость затухания при включении
#define RETURN_SPEED       15000     //скорость возврата к первому каналу регулировки
#define SPEED_FLASH_RETURN 50        //скорость вспышек при возврате и переключении
#define SAVE_SPEED         30000     //время задержки перед сохранением в eeprom


volatile unsigned char flag;     //переменная для хранения флагов событий
#define RETURN_START 0           //бит флага отсчета времени для возврата на первый канал регулировки
#define RETURN_READY 1           //бит готовности возврата на первый канал регулировки
#define SAVE_START   2           //бит флага отсчета времени для сохранения данных в еепром
#define SAVE_READY   3           //бит готовности сохранения данных в еепром


unsigned char bright=250;        //максимальная яркость каналов




ISR(TIMER0_OVF_vect)             //Обработчик прерывания по переполнению таймера TC0
{
/*      процедура опроса энкодера   */
NewState=PIND & 0b00000011;
if(NewState!=OldState)
{
	switch(OldState)
	{
		case 2:
		{
			if(NewState == 3) upState++;
			if(NewState == 0) downState++;
			break;
		}
		
		case 0:
		{
			if(NewState == 2) upState++;
			if(NewState == 1) downState++;
			break;
		}
		case 1:
		{
			if(NewState == 0) upState++;
			if(NewState == 3) downState++;
			break;
		}
		case 3:
		{
			if(NewState == 1) upState++;
			if(NewState == 2) downState++;
			break;
		}
	}
	OldState=NewState;
}
/*********************************************/  


if ((flag&(1<<RETURN_START))!=0)               //обработка задержки возврата к регулировке яркости шкал
	{
		ReturnCounter++;
		
		if (ReturnCounter==RETURN_SPEED)
			{
				flag|=(1<<RETURN_READY);
				flag&=(~(1<<RETURN_START));
				ReturnCounter=0;
			}
	}

if ((flag&(1<<SAVE_START))!=0)                //обработка задержки сохранения данных в EEPROM
	{
		SaveCounter++;
		
		if (SaveCounter==SAVE_SPEED)
			{
				flag|=(1<<SAVE_READY);
				flag&=(~(1<<SAVE_START));
				SaveCounter=0;
			}
	}

	
}



void Save (void)                          //функция сохранения данных в EEPROM
{
	red1=RED1;green1=GREEN1;blue1=BLUE1;
	scales=SCALES;
	
	flag&=(~(1<<SAVE_READY));
	eeprom_write_byte(&e_tmp, tmp);
	eeprom_write_byte(&e_red1, red1);
	eeprom_write_byte(&e_green1, green1);
	eeprom_write_byte(&e_blue1, blue1);
	eeprom_write_byte(&e_scales, scales);
	
	
	
}


void return_set (void)                    //функция возвращения к регулировке яркости шкал
{
	set=0;
	flag&=(~(1<<RETURN_READY));
	scales=SCALES;
	SCALES=0;
	_delay_ms(SPEED_FLASH_RETURN);
	SCALES=scales;
}



void encoder_r(void)                      //обработка вращения энкодера вправо
{
	if(set==0)
	{
		if (SCALES<bright) {SCALES+=STEP;}
	}
	
	if(set==1)
	{	
		if (tmp==1) {if (GREEN1<bright) GREEN1+=STEP; else tmp=2;}
		if (tmp==2) {if (RED1>0)        RED1-=STEP;   else tmp=3;}
		if (tmp==3) {if (BLUE1<bright)  BLUE1+=STEP;  else tmp=4;}
		if (tmp==4) {if (GREEN1>0)      GREEN1-=STEP; else tmp=5;}
		if (tmp==5) {if (RED1<bright)   RED1+=STEP;   else tmp=6;}
		if (tmp==6) {if (BLUE1>0)       BLUE1-=STEP;  else tmp=1;}
		}
		
if (OCR0B==0) {DDRD&=(~(1<<5));} else {DDRD|=(1<<5);}  //Проверяем значения каналов ШИМ, если оно равно нулю-
if (OCR0A==0) {DDRD&=(~(1<<6));} else {DDRD|=(1<<6);}  //перенастраиваем порты на вход, для полного гашения светодиодов
if (OCR1AL==0){DDRB&=(~(1<<1));} else {DDRB|=(1<<1);}
if (OCR1BL==0){DDRB&=(~(1<<2));} else {DDRB|=(1<<2);}
}


void encoder_l(void)                      //обработка вращения энкодера влево
{
	if(set==0)
	{
		if(SCALES>0) {SCALES-=STEP;}
	}
		
	if(set==1)
	{
		if (tmp == 1) {if(GREEN1>0)      GREEN1-=STEP; else tmp = 6;}
		if (tmp == 2) {if(RED1<bright)   RED1+=STEP;   else tmp = 1;}
		if (tmp == 3) {if(BLUE1>0)       BLUE1-=STEP;  else tmp = 2;}
		if (tmp == 4) {if(GREEN1<bright) GREEN1+=STEP; else tmp = 3;}
		if (tmp == 5) {if(RED1>0)        RED1-=STEP;   else tmp = 4;}
		if (tmp == 6) {if(BLUE1<bright)  BLUE1+=STEP;  else tmp = 5;}
		}
		
if (OCR0B==0) {DDRD&=(~(1<<5));} else {DDRD|=(1<<5);}  //Проверяем значения каналов ШИМ, если оно равно нулю-
if (OCR0A==0) {DDRD&=(~(1<<6));} else {DDRD|=(1<<6);}  //перенастраиваем порты на вход, для полного гашения светодиодов
if (OCR1AL==0){DDRB&=(~(1<<1));} else {DDRB|=(1<<1);}
if (OCR1BL==0){DDRB&=(~(1<<2));} else {DDRB|=(1<<2);}
}


int main(void)
{


/* Настройки таймеров на генерацию восьмибитного ШИМ */	
TCCR0A|=((1<<WGM01)|(1<<WGM00)|(1<<COM0A1)|(1<<COM0B1)); 
TCCR0B|=(1<<CS01);

TCCR1A|=((1<<WGM10)|(1<<COM1A1)|(1<<COM1B1));
TCCR1B|=((1<<CS11)|(1<<WGM12));

TCCR2A|=((1<<WGM01)|(1<<WGM00)|(1<<COM0A1)|(1<<COM0B1));
TCCR2B|=(1<<CS01);

/* Прерывание по переполнению таймера TC0*/
TIMSK0|=(1<<TOIE0);

/* Конфигурация портов  */
DDRD|=((1<<5)|(1<<6)|(1<<3));
DDRB|=((1<<3)|(1<<1)|(1<<2));

PORTD|=((1<<0)|(1<<1));
PORTB|=(1<<6);

/* Извлечение данных из EEPROM  */
scales = eeprom_read_byte(&e_scales);     
red1   = eeprom_read_byte(&e_red1);
green1 = eeprom_read_byte(&e_green1);
blue1  = eeprom_read_byte(&e_blue1);
tmp    = eeprom_read_byte(&e_tmp);

/* Плавный розжиг */
while(SCALES<255)        {SCALES++; _delay_ms(LIGHT_UP);};
while((RED1<red1)|(GREEN1<green1)|(BLUE1<blue1)) 
	{
		if (RED1<red1)     {RED1++;}
		if (GREEN1<green1) {GREEN1++;}
		if (BLUE1<blue1)   {BLUE1++;}
		_delay_ms(LIGHT_UP);
	}

/* Плавное затухание после розжига */
while(SCALES>scales)        {SCALES--; _delay_ms(LIGHT_DOWN);};
RED1=red1; GREEN1=green1; BLUE1= blue1;



sei();


    while(1)
    {
	
	/* обработка правого вращения энкодера */
	if (upState >= 4)
	{
		encoder_r();
		upState = 0;
		ReturnCounter = 0;
		SaveCounter   = 0;
		flag|=(1<<SAVE_START);
	}
	
	/* Обработка левого вращения энкодера */
	if (downState >= 4)
	{
		encoder_l();
		downState = 0;
		ReturnCounter = 0;
		SaveCounter   = 0;
		flag|=(1<<SAVE_START);
	}
	
	/* обработка нажатия кнопки */
	if ((PINB&(1<<6))==0)
	{
		ReturnCounter = 0;
		SaveCounter   = 0;
		
		_delay_ms(10);
		while((PINB&(1<<6))==0){}
		set++;
		if(set>1){set=0;}
		
		if(set!=0)
			{
				/* Моргание стрелок */
				red1=RED1;green1=GREEN1;blue1=BLUE1;
				RED1=0;GREEN1=0;BLUE1=0;
				_delay_ms(SPEED_FLASH_RETURN);
				RED1=red1;GREEN1=green1;BLUE1=blue1;
				_delay_ms(SPEED_FLASH_RETURN);
				RED1=0;GREEN1=0;BLUE1=0;
				_delay_ms(SPEED_FLASH_RETURN);
				RED1=red1;GREEN1=green1;BLUE1=blue1;
				flag|=(1<<RETURN_START); //запуск таймера возврата к регулировке шкал
				ReturnCounter=0;
			}
		else 
			{
				/* Моргание шкал */
				scales=SCALES;
				SCALES=0;
				_delay_ms(SPEED_FLASH_RETURN);
				SCALES=scales;
				_delay_ms(SPEED_FLASH_RETURN);
				SCALES=0;
				_delay_ms(SPEED_FLASH_RETURN);
				SCALES=scales;
				flag&=(~(1<<RETURN_START)); //остановка таймера возврата к регулировке шкал 
				ReturnCounter=0;
			}
		
		_delay_ms(1);
	}
	
	if ((flag&(1<<RETURN_READY))!=0) {return_set();} //проверка готовности к возврату к регулировке шкал
	if ((flag&(1<<SAVE_READY))!=0)   {Save();}		 //проверка готовности к сохранению данных в EEPROM
	
	
	
    }
}